/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user.common.errorhandling;

import jakarta.inject.Inject;
import jakarta.ws.rs.NotFoundException;
import jakarta.ws.rs.core.Response;
import jakarta.ws.rs.core.Response.Status;
import jakarta.ws.rs.core.UriInfo;

import org.jboss.resteasy.reactive.server.ServerExceptionMapper;

import lombok.extern.log4j.Log4j2;

@Log4j2
public class ExceptionHandler {

	@Inject
	ApiErrorUtil apiErrorUtil;

	@Inject
	UriInfo uriInfo;

	@ServerExceptionMapper
	public Response mapRuntimeException(RuntimeException e) {
		LOG.error(e.getMessage(), e);
		return Response.status(Status.INTERNAL_SERVER_ERROR)
				.entity(apiErrorUtil.buildFromRuntimeException(e))
				.build();
	}

	@ServerExceptionMapper
	public Response mapNotFoundException(NotFoundException e) {
		LOG.warn(e.getMessage() + ". Path: " + uriInfo.getPath(), e);
		return Response.status(Status.NOT_FOUND)
				.entity(apiErrorUtil.buildFromNotFoundException(e))
				.build();
	}

	@ServerExceptionMapper
	public Response mapResourceNotFoundException(ResourceNotFoundException e) {
		LOG.info(e.getMessage(), e);
		return Response.status(Status.NOT_FOUND)
				.entity(apiErrorUtil.buildFromResourceNotFoundException(e))
				.build();
	}

	@ServerExceptionMapper
	public Response mapAccessForbiddenException(AccessForbiddenException e) {
		LOG.error(e.getMessage(), e);
		return Response.status(Status.FORBIDDEN)
				.entity(apiErrorUtil.buildFromAccessForbiddenException(e))
				.build();
	}

	@ServerExceptionMapper
	public Response mapKeycloakUnavailableException(KeycloakUnavailableException e) {
		LOG.error(e.getMessage(), e);
		return Response.status(Status.SERVICE_UNAVAILABLE)
				.entity(apiErrorUtil.buildFromKeycloakUnavailableException(e))
				.build();
	}
}