/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user;

import java.util.List;
import java.util.Map;

import jakarta.ws.rs.core.Response;

import org.keycloak.admin.client.resource.RoleMappingResource;
import org.keycloak.admin.client.resource.RoleScopeResource;
import org.keycloak.admin.client.resource.UserResource;
import org.keycloak.representations.idm.ClientMappingsRepresentation;
import org.keycloak.representations.idm.CredentialRepresentation;
import org.keycloak.representations.idm.FederatedIdentityRepresentation;
import org.keycloak.representations.idm.GroupRepresentation;
import org.keycloak.representations.idm.MappingsRepresentation;
import org.keycloak.representations.idm.RoleRepresentation;
import org.keycloak.representations.idm.UserRepresentation;
import org.keycloak.representations.idm.UserSessionRepresentation;

import lombok.NoArgsConstructor;

@NoArgsConstructor
class UserResourceStub implements UserResource {

	private UserRepresentation userRepresentation = UserRepresentationTestFactory.create();
	private List<GroupRepresentation> groups = List.of(GroupRepresentationTestFactory.createGroup(UserResourceMapperTest.GROUP_1_PATH));

	public UserResourceStub(Map<String, List<String>> attributes) {
		userRepresentation = UserRepresentationTestFactory.createWithAttributes(attributes);
	}

	public UserResourceStub(List<GroupRepresentation> groups) {
		this.groups = groups;
	}

	@Override
	public UserRepresentation toRepresentation() {
		return userRepresentation;
	}

	@Override
	public UserRepresentation toRepresentation(boolean userProfileMetadata) {
		return userRepresentation;
	}

	@Override
	public void update(UserRepresentation userRepresentation) {
		// not implemented
	}

	@Override
	public void remove() {
		// not implemented
	}

	@Override
	public List<GroupRepresentation> groups() {
		return groups;
	}

	@Override
	public void leaveGroup(String groupId) {
		// not implemented
	}

	@Override
	public List<GroupRepresentation> groups(Integer firstResult, Integer maxResults) {
		return groups();
	}

	@Override
	public List<GroupRepresentation> groups(String search, Integer firstResult, Integer maxResults) {
		return groups();
	}

	@Override
	public List<GroupRepresentation> groups(Integer firstResult, Integer maxResults, boolean briefRepresentation) {
		return groups();
	}

	@Override
	public List<GroupRepresentation> groups(String search, boolean briefRepresentation) {
		return groups();
	}

	@Override
	public List<GroupRepresentation> groups(String search, Integer firstResult, Integer maxResults, boolean briefRepresentation) {
		return groups();
	}

	@Override
	public Map<String, Long> groupsCount(String search) {
		return Map.of();
	}

	@Override
	public void joinGroup(String groupId) {
		// not implemented
	}

	@Override
	public void logout() {
		// not implemented
	}

	@Override
	public List<CredentialRepresentation> credentials() {
		return null;
	}

	@Override
	public List<String> getConfiguredUserStorageCredentialTypes() {
		return null;
	}

	@Override
	public void removeCredential(String credentialId) {
		// not implemented
	}

	@Override
	public void setCredentialUserLabel(String credentialId, String userLabel) {
		// not implemented
	}

	@Override
	public void moveCredentialToFirst(String credentialId) {
		// not implemented

	}

	@Override
	public void moveCredentialAfter(String credentialId, String newPreviousCredentialId) {
		// not implemented
	}

	@Override
	public void disableCredentialType(List<String> credentialTypes) {
		// not implemented
	}

	@Override
	public void resetPassword(CredentialRepresentation credentialRepresentation) {
		// not implemented
	}

	@Override
	public void resetPasswordEmail() {
		// not implemented
	}

	@Override
	public void resetPasswordEmail(String clientId) {
		// not implemented
	}

	@Override
	public void executeActionsEmail(List<String> actions) {
		// not implemented
	}

	@Override
	public void executeActionsEmail(List<String> actions, Integer lifespan) {
		// not implemented
	}

	@Override
	public void executeActionsEmail(String clientId, String redirectUri, Integer lifespan, List<String> actions) {
		// not implemented
	}

	@Override
	public void executeActionsEmail(String clientId, String redirectUri, List<String> actions) {
		// not implemented
	}

	@Override
	public void sendVerifyEmail() {
		// not implemented
	}

	@Override
	public void sendVerifyEmail(String clientId) {
		// not implemented
	}

	@Override
	public List<UserSessionRepresentation> getUserSessions() {
		return null;
	}

	@Override
	public List<UserSessionRepresentation> getOfflineSessions(String clientId) {
		return null;
	}

	@Override
	public List<FederatedIdentityRepresentation> getFederatedIdentity() {
		return null;
	}

	@Override
	public Response addFederatedIdentity(String provider, FederatedIdentityRepresentation rep) {
		return null;
	}

	@Override
	public void removeFederatedIdentity(String provider) {
		// not implemented
	}

	@Override
	public RoleMappingResource roles() {
		return new RoleMappingResource() {

			@Override
			public RoleScopeResource realmLevel() {
				return null;
			}

			@Override
			public MappingsRepresentation getAll() {
				var rep = new MappingsRepresentation();
				var clientMapRep = new ClientMappingsRepresentation();
				var roleRep = new RoleRepresentation(UserRepresentationTestFactory.ROLE_NAME, "Test role", false);
				clientMapRep.setMappings(List.of(roleRep));
				rep.setClientMappings(Map.of("alfa", clientMapRep));
				return rep;
			}

			@Override
			public RoleScopeResource clientLevel(String clientUUID) {
				return null;
			}
		};
	}

	@Override
	public List<Map<String, Object>> getConsents() {
		return List.of(Map.of());
	}

	@Override
	public void revokeConsent(String clientId) {
		// not implemented
	}

	@Override
	public Map<String, Object> impersonate() {
		return null;
	}

	@Override
	public void sendVerifyEmail(String clientId, String redirectUri) {
		// not implemented
	}

	@Override
	public void sendVerifyEmail(Integer lifespan) {
		// not implemented
	}

	@Override
	public void sendVerifyEmail(String clientId, String redirectUri, Integer lifespan) {
		// not implemented
	}
}
