package de.ozgcloud.user.organisationseinheit;

import static org.assertj.core.api.Assertions.*;

import de.ozgcloud.user.TestDatabaseUtils;
import jakarta.inject.Inject;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import com.mongodb.client.MongoClient;

import de.ozgcloud.user.UserTestFactory;
import de.ozgcloud.user.common.MongoDbTestProfile;
import io.quarkus.test.junit.QuarkusTest;
import io.quarkus.test.junit.TestProfile;

@QuarkusTest
@TestProfile(MongoDbTestProfile.class)
class OrganisationsEinheitRepositoryITCase {

	@Inject
	OrganisationsEinheitRepository repository;

	@Inject
	MongoClient mongoClient;

	@Nested
	@DisplayName("Test finding all Organisationseinheiten IDs")
	class TestFindAllOrganisationsEinheitIds {

		@BeforeEach
		void init() {
			var mongoDb = TestDatabaseUtils.getDatabase(mongoClient);
			mongoDb.getCollection("User").drop();
		}

		@Test
		void shouldNotLoadDeletedUser() {
			repository.persist(UserTestFactory.createBuilder().deleted(true).build());

			var result = repository.findAllOrganisationsEinheitIds();

			assertThat(result).isEmpty();
		}

		@Test
		void shouldLoadDistinct() {
			repository.persist(UserTestFactory.create());
			repository.persist(UserTestFactory.createBuilder().id(null).build());

			var result = repository.findAllOrganisationsEinheitIds();

			assertThat(result)
					.hasSize(1)
					.containsExactlyInAnyOrder(UserTestFactory.ORGANISTATIONSEINHEITEN_ID);
		}
	}

}